# $Id: snmp.tcl,v 2.3 2001/02/06 15:52:50 gdmr Exp $
#
# Copyright 2001 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# SNMP-based procedures to query the switches.  Cache the results
# where appropriate.
#

####################
# Initialisation
####################

puts stderr "Loading Tnm..."
package require Tnm 3.0
namespace import Tnm::snmp
namespace import Tnm::mib

# To speed up loading we pare things down to the bare minimum
set tnm(mibs) "BRIDGE-MIB"
set mibList {}

#puts stderr "MIBs: $mibList"
#foreach m $mibList {
#   mib load [file join [file dirname $argv0] $m]
#}

set snmpRetries 1
set snmpTimeout 30

proc snmpInit {host community} {
   global snmpS
   global snmpRetries
   global snmpTimeout
   set snmpS [snmp generator \
               -address $host -community $community \
               -retries $snmpRetries -timeout $snmpTimeout]
}

####################
# System group
####################

proc snmpDescr {snmpS} {
   set vb "SNMPv2-MIB!sysDescr.0"
   set vb [$snmpS get $vb]
   eval set snmpSysDescr [snmp value $vb]
}

proc snmpUpTime {snmpS} {
   set vb "SNMPv2-MIB!sysUpTime.0"
   set vb [$snmpS get $vb]
   expr {[snmp value $vb] / 100}
}

proc snmpLocation {snmpS} {
   set vb "SNMPv2-MIB!sysLocation.0"
   set vb [$snmpS get $vb]
   eval set snmpSysLocation [snmp value $vb]
}

####################
# Interfaces
####################

#proc snmpEtherIfIndices {} {
#   global snmpS
#   global ifNameToIndex
#   global ifIndexToName
#   set ifRoot [mib oid "IF-MIB!ifIndex"]
#   set vbl "$ifRoot.0
#            IF-MIB!ifType.0
#            IF-MIB!ifDescr.0"
#   while {1} {
#      set vbl [$snmpS getnext $vbl]
#      if {! [mib subtree $ifRoot [snmp oid $vbl 0]]} break
#      if {[snmp value $vbl 1] == "ethernetCsmacd"} {
#         lappend indices [snmp value $vbl 0]
#         set i [snmp value $vbl 0]
#         set n [snmp value $vbl 2]
#         set ifNameToIndex($n) $i
#         set ifIndexToName($i) $n
#      }
#   }
#   # Don't need to sort, they'll be in numeric order already
#   set indices
#}

#proc snmpIfName {i} {
#   global ifIndexToName
#   if [info exists ifIndexToName($i)] {
#      set ifIndexToName($i)
#   } else {
#      return "*unknown $i*"
#   }
#}

#proc snmpIfIndex {n} {
#   global ifNameToIndex
#   if [info exists ifNameToIndex($n)] {
#      set ifNameToIndex($n)
#   } else {
#      return -1
#   }
#}
#proc snmpIfType {i} {
#   global snmpIfTypes
#   global snmpS
#   if {[info exists snmpIfTypes($i)]} {
#      set snmpIfTypes($i)
#   } else {
#      set vb "IF-MIB!ifType.$i"
#      set vb [$snmpS get $vb]
#      set snmpIfTypes($i) [snmp value $vb]
#   }
#}

#proc snmpIfDescr {i} {
#   global snmpIfDescrs
#   global snmpS
#   if {[info exists snmpIfDescrs($i)]} {
#      set snmpIfDescrs($i)
#   } else {
#      set vb "IF-MIB!ifDescr.$i"
#      set vb [$snmpS get $vb]
#      eval set snmpIfDescrs($i) [snmp value $vb]
#   }
#}

#proc snmpIfUp {i} {
#   global snmpS
#   set vb "IF-MIB!ifOperStatus.$i"
#   set vb [$snmpS get $vb]
#   if {[snmp value $vb] == "up"} {
#      return 1
#   } else {
#      return 0
#   }
#}

#proc snmpIfAdmin {i} {
#   global snmpS
#   set vb "IF-MIB!ifAdminStatus.$i"
#   set vb [$snmpS get $vb]
#   if {[snmp value $vb] == "up"} {
#      return 1
#   } else {
#      return 0
#   }
#}

#proc snmpIfSpeed {i} {
#   global snmpS
#   set vb "IF-MIB!ifSpeed.$i"
#   set vb [$snmpS get $vb]
#   snmp value $vb
#}

#proc snmpIfLastChange {i} {
#   global snmpS
#   set vb "IF-MIB!ifLastChange.$i"
#   set vb [$snmpS get $vb]
#   snmp value $vb
#}

#proc snmpIfStats {i} {
#   global snmpS
#   set vbl "IF-MIB!ifInOctets.$i
#            IF-MIB!ifInUcastPkts.$i
#            IF-MIB!ifInErrors.$i
#            IF-MIB!ifOutOctets.$i
#            IF-MIB!ifOutUcastPkts.$i
#            IF-MIB!ifOutErrors.$i"
#   set vbl [$snmpS get $vbl]
#   snmp value $vbl
#}

##########
# Port forwarding table
##########

proc snmpPortFwdTable {snmpS} {
   set sList {}
   set sRoot [mib oid "BRIDGE-MIB!dot1dTpFdbAddress"]
   set vbl "$sRoot.0
            BRIDGE-MIB!dot1dTpFdbPort.0"
   while {1} {
      set vbl [$snmpS getnext $vbl]
      if {! [mib subtree $sRoot [snmp oid $vbl 0]]} break
      lappend sList [snmp value $vbl]
   }
   set sList
}

##########
# Etherlike counters
##########

#proc snmpPortEtherLikeErrors {snmpS i} {
#   set vbl "EtherLike-MIB!dot3StatsAlignmentErrors.$i
#            EtherLike-MIB!dot3StatsFCSErrors.$i
#            EtherLike-MIB!dot3StatsSingleCollisionFrames.$i
#            EtherLike-MIB!dot3StatsMultipleCollisionFrames.$i
#            EtherLike-MIB!dot3StatsDeferredTransmissions.$i
#            EtherLike-MIB!dot3StatsLateCollisions.$i
#            EtherLike-MIB!dot3StatsExcessiveCollisions.$i
#            EtherLike-MIB!dot3StatsInternalMacTransmitErrors.$i
#            EtherLike-MIB!dot3StatsCarrierSenseErrors.$i
#            EtherLike-MIB!dot3StatsFrameTooLongs.$i"
#   set vbl [$snmpS get $vbl]
#   snmp value $vbl
#}
#
#proc snmpPortEtherLikeErrorsLabels {} {
#   return {Align FCS SColl MColl Defer Late Excsv IMacTX CSense TooLong}
#}
#
#proc snmpPortEtherLikeErrorsDispflags {} {
#   return {1 1 -1 -1 -1 1 1 1 0 1}
#}
